// $Id: RMOQuickPhotonPropagator.cxx,v 1.4 2004/10/04 03:34:36 hoshina Exp $
//*************************************************************************
//* --------------------
//* RMOQuickPhotonPropagator
//* --------------------
//* (Description)
//*     Quick photon propagator from the surface of glass to 
//*     the surface of PMT.
//*     The imput photons are spread on the PMT surface under
//*     the arrival map of photons generated by DOMINANT.
//*     Note that THE MODULE DOES NOT ELIMINATE ANY PHOTON.
//*     Number of the imput photons must be calcurated with
//*     taking into account of absorption at glass and gel, 
//*     reflection and refraction, and angle acceptance.
//*    
//* (Requires)
//*     RMOOpticalPhoton
//*     RMOPhotonBuf
//* (Provides)
//*     RMOQuickPhotonPropagator              : Quick photon propagator module
//* (Update Record)
//*	2004/09/22  K.Hoshina	Original version.
//*************************************************************************

#include "RMOQuickPhotonPropagator.h"
#include "RMOPhotonBuf.h"
#include "RMOOpticalPhoton.h"
#include "RMOUnit.h"
#include "TRandom.h"
#include "TROOT.h"

#include <iostream>
#include <fstream>


ClassImp(RMOQuickPhotonPropagator)

using namespace std;

Double_t RMOQuickPhotonPropagator::fCenter_of_pmt_z = (0.84 * kMILLI_METER);
TString  RMOQuickPhotonPropagator::fCalibFileName = "QuickPhotonPropagatorMap.data";
TString  RMOQuickPhotonPropagator::fCalibRootFileName = "QuickPhotonPropagatorMap.root";

//=====================================================================
//* constructor -------------------------------------------------------

RMOQuickPhotonPropagator::RMOQuickPhotonPropagator()
                   : RMOVModule("RMOQuickPhotonPropagator","quick photon propagator"),
                     fRefHistName("surv_13inch_0deg_map_3D"), fCalibFile(0),
                     fPhotonHist(0), fOutputPhotonBuf(0), fCanvas(0)
{
   SetCalibFileNames(TString("QuickPhotonPropagatorMap.data"));
}

RMOQuickPhotonPropagator::RMOQuickPhotonPropagator(const TString &name)
                   : RMOVModule("RMOQuickPhotonPropagator","quick photon propagator"),
                     fRefHistName("surv_13inch_0deg_map_3D"), fCalibFile(0),
                     fPhotonHist(0), fOutputPhotonBuf(0), fCanvas(0)
{
   SetCalibFileNames(name);
}

//=====================================================================
//* destructor --------------------------------------------------------

RMOQuickPhotonPropagator::~RMOQuickPhotonPropagator()
{
//*-*-*-*-*-*-*-*-*-*-*-*Destructor*-*-*-*-*-*-*-*-*-*-*-*-*
//                       ==========
  if (fPhotonHist)      delete fPhotonHist;
  if (fOutputPhotonBuf) delete fOutputPhotonBuf;
  if (fCanvas)          delete fCanvas;
}

//===================================================================
//* Configure ---------------------------------------------------------

void RMOQuickPhotonPropagator::Configure()
{
}

//=====================================================================
//* Process -----------------------------------------------------------

void RMOQuickPhotonPropagator::Process()
{
//*-*-*-*-*-*-*-*-*-*-*-*Process*-*-*-*-*-*-*-*-*-*-*-*-*
//                       =======

   fCalibFile = TFile::Open(fCalibRootFileName, "read");
   TString dirname(fCalibRootFileName);
   dirname += ":/";

   /* Get Input Data --------------------- */

   RMOPhotonBuf imputbuf = (RMOPhotonBuf &)GetImputBuffer();
   TIter        iter  = imputbuf.GetIter();

   /* Convert pe to charge --------------- */
   if (fOutputPhotonBuf) delete fOutputPhotonBuf;

   fOutputPhotonBuf = new RMOPhotonBuf();
   fOutputPhotonBuf->SetOwner();

   RMOOpticalPhoton *iph;
   while ((Int_t)(iph = (RMOOpticalPhoton *)iter.Next())) {
      TVector3 phdir   = iph->GetDirection();
      TVector3 phpos   = GetRandomPosition(iph, dirname, kTRUE, GetVerboseLevel());
      RMOOpticalPhoton *oph = new RMOOpticalPhoton(phpos, phdir);

      // TMP_TREATMENT!! <<<<<<<<<<<
      // set local position 
      
      Double_t wavelen = iph->GetWaveLength();
      oph->SetWaveLength(wavelen);

      TVector3 localpos(phpos.X(), phpos.Y(), phpos.Z() - fCenter_of_pmt_z);
      Double_t costheta = localpos.CosTheta();
      Double_t phi      = localpos.Phi();
      while (phi < 0)              phi += TMath::TwoPi();    // 0 < phi < 2*pi 
      while (phi > TMath::TwoPi()) phi -= TMath::TwoPi();

      oph->SetCosTheta(costheta);
      oph->SetAzimuth(phi); 
      if (GetVerboseLevel() >= kDEBUG) {
         cerr << kDEBUG << " "
              << "localpos (" << localpos.X() << " " << localpos.Y() 
                             << " " << localpos.Z() << ")" << endl;
         cerr << kDEBUG << " "
              << "costheta, azimus = " << costheta << " " << phi << endl;
      }
      oph->SetMother(iph);
      fOutputPhotonBuf->Add(oph);

   }

   fCalibFile->Close();
   gDirectory->cd("/");
   
}

//=====================================================================
//* Finish ------------------------------------------------------------

void RMOQuickPhotonPropagator::Finish()
{

}

//=====================================================================
//* Draw --------------------------------------------------------------

void RMOQuickPhotonPropagator::Draw()
{
   fCalibFile = TFile::Open(fCalibRootFileName, "read");
   TString dirname(fCalibRootFileName);
   dirname += ":/";
   gDirectory->cd(dirname.Data());

   TH3D    *refhist = (TH3D*)gROOT->FindObject(fRefHistName);
   Int_t    nxbins = refhist->GetNbinsX();
   Int_t    nybins = refhist->GetNbinsY();
   Int_t    nzbins = refhist->GetNbinsZ();
   Double_t xmin = refhist->GetXaxis()->GetXmin();
   Double_t xmax = refhist->GetXaxis()->GetXmax();
   Double_t ymin = refhist->GetYaxis()->GetXmin();
   Double_t ymax = refhist->GetYaxis()->GetXmax();
   Double_t zmin = refhist->GetZaxis()->GetXmin();
   Double_t zmax = refhist->GetZaxis()->GetXmax();
   
   fCalibFile->Close();
   gDirectory->cd("/");

   if (fCanvas)       delete fCanvas;
   if (fPhotonHist)   delete fPhotonHist;

   fPhotonHist = new TH3D("Hit_photons", "Hit_photons",
                           nxbins, xmin, xmax,
                           nybins, ymin, ymax,
                           nzbins, zmin, zmax);

   fPhotonHist->GetXaxis()->SetTitle("x [m]");
   fPhotonHist->GetYaxis()->SetTitle("y [m]");
   fPhotonHist->GetZaxis()->SetTitle("z [m]");
   
   TIter iter = fOutputPhotonBuf->GetIter();
   RMOOpticalPhoton *datum;
   while ((Int_t)(datum = (RMOOpticalPhoton *)iter.Next())) {
      TVector3 pos = datum->GetPosition();
      fPhotonHist->Fill(pos.X(), pos.Y(), pos.Z());
   }
      
   fCanvas  = new TCanvas("Hit_photons", "Hit photons on PMT",
                          10, 10, 800, 400);
   fCanvas->Divide(2, 1);
   fCanvas->cd(1); fPhotonHist->Draw("BOX");
   TH1 *projected = fPhotonHist->Project3D("yx");
   projected->GetXaxis()->SetTitle("x [m]");
   projected->GetYaxis()->SetTitle("y [m]");
   fCanvas->cd(2); projected->Draw("LEGO2");

}

//=====================================================================
//* GetRandomPosition -------------------------------------------------

TVector3 RMOQuickPhotonPropagator::GetRandomPosition(RMOOpticalPhoton *iph,
                                                     const TString &dirname,
                                                     Bool_t isglobal,
                                                     EVerboseLevel verbose)
{

   gDirectory->cd(dirname.Data());

   if (verbose >= kFUNCTION_CALLED) {
      cerr << kFUNCTION_CALLED << " " 
           << "RMOQuickPhotonPropagator::GetRandomPosition is called!" << endl;
   }

   // digitize angle

   TVector3 dir = iph->GetDirection();  // dir must point to the center of OM
   dir = -dir;   
   Double_t theta = dir.Theta();
   Double_t phi   = dir.Phi();

   if (verbose >= kDEBUG) {
      cerr << kDEBUG << " " 
           << "direction(" << dir.X() << ", " << dir.Y() << ", " << dir.Z() << ")  "
           << "theta : " << theta / kDEGREE  << "deg  " 
           << "phi : " << phi / kDEGREE << "deg"<< endl;
   }

   Int_t itheta = -5000;

   if (theta < 5 * kDEGREE) {
      itheta = 0;  
   } else if (theta < 15 * kDEGREE) {
      itheta = 10;  
   } else if (theta < 25 * kDEGREE) {
      itheta = 20;  
   } else if (theta < 35 * kDEGREE) {
      itheta = 30;  
   } else if (theta < 45 * kDEGREE) {
      itheta = 40;  
   } else if (theta < 55 * kDEGREE) {
      itheta = 50;  
   } else if (theta < 65 * kDEGREE) {
      itheta = 60;  
   } else if (theta < 75 * kDEGREE) {
      itheta = 70;  
   } else if (theta < 85 * kDEGREE) {
      itheta = 80;  
   } else if (theta < 95 * kDEGREE) {
      itheta = 90;  
   } else if (theta < 105 * kDEGREE) {
      itheta = 100;  
   } else if (theta < 115 * kDEGREE) {
      itheta = 110;  
   } else if (theta < 125 * kDEGREE) {
      itheta = 120;  
   } else if (theta < 135 * kDEGREE) {
      itheta = 130;  
   } else if (theta < 145 * kDEGREE) {
      itheta = 140;  
   } else if (theta < 155 * kDEGREE) {
      itheta = 150;  
   } else if (theta < 165 * kDEGREE) {
      itheta = 160;  
   } else if (theta < 175 * kDEGREE) {
      itheta = 170;  
   } else if (theta < 185 * kDEGREE) {
      itheta = 180;  
   } else {
      cerr << "RMOQuickPhotonPropagator::GetRandomPosition:: Angle Range Over! abort." << endl;
      abort();
   }

   TString histname("surv_13inch_");
   histname += itheta;
   histname += "deg_map_3D";

   if (verbose >= kDEBUG) {
      cerr << kDEBUG << " " 
           << "calib file name is " << histname << endl;
   }

   TH3D *curmap = (TH3D *)gROOT->FindObject(histname);
   Double_t x, y, z;
   curmap->GetRandom3(x, y, z);
   TVector3 localpos(x, y, z);

   // rotate local point to global point

   localpos.RotateZ(phi);

   gDirectory->cd("/");

   if (isglobal == kFALSE) {
      localpos.SetZ(localpos.Z() - fCenter_of_pmt_z);
   }

   return localpos;
   
}

//=====================================================================
//* SetCalibFileNames --------------------------------------

void RMOQuickPhotonPropagator::SetCalibFileNames(const TString &name)
{
   fCalibFileName = name;
   fCalibRootFileName = name;
   fCalibRootFileName.Remove(name.Index("."));
   fCalibRootFileName += ".root";
}

//=====================================================================
//* MakeQuickPhotonPropagatorMap --------------------------------------

void RMOQuickPhotonPropagator::MakeQuickPhotonPropagatorMap(const TString &fname)
{
   ifstream in(fname.Data());

   TString outname(fname);
   outname.Remove(fname.Index("."));
   outname += ".root";

   cerr << "outname = " << outname << endl;
   
   TFile file(outname.Data(), "recreate");

   Int_t     nx, ny, nz, ibin;
   Double_t  xmin, xmax, ymin, ymax, zmin, zmax;
   Double_t  dum, x, y, z, content;
   TString   histname;
   TObjArray map3Darray;
   //map3Darray.SetOwner();

   in >> nx >> xmin >> xmax;
   in >> ny >> ymin >> ymax;
   in >> nz >> zmin >> zmax;

   TH3D *hMap = 0;
   while (in >> dum) {

      if (dum == -99999999) {
         in >> histname;
         cerr << "create new hist " << histname << endl;
         hMap = new TH3D(histname, histname, nx * 2, xmin, xmax,
                                             ny * 2, ymin, ymax,
                                             nz, zmin, zmax);
         map3Darray.Add(hMap);          

      } else {

         x = dum;
         in >> y >> z >> ibin >> content;

         hMap->Fill(x, y, z, content);

      }
   }
   in.close();
   in.clear();

   file.Write();
   file.Close("R");
   
}         
 
  

