// $Id: RMOWaveLengthSimulator.cxx,v 1.1 2004/09/26 03:45:37 hoshina Exp $
//*************************************************************************
//* --------------------
//* RMOWaveLengthSimulator
//* --------------------
//* (Description)
//*     It simulates wave length spectrum of Cerenkov photons
//*     after X[m] propagation in Antiarctic ice.
//*
//* (Requires)
//*     RMOMCPhotonHitDatum
//*     RMOMCPhotonHitBuf
//* (Provides)
//*     RMOWaveLengthSimulator   : wave length simulation module
//* (Update Record)
//*	2004/09/22  K.Hoshina	Original version.
//*************************************************************************

#include "RMOWaveLengthSimulator.h"
#include "RMOMCPhotonHitBuf.h"
#include "RMOMCPhotonHit.h"
#include "RMOPhotonBuf.h"
#include "RMOOpticalPhoton.h"
#include "RMOUnit.h"
#include "TRandom.h"

#include <iostream>
#include <fstream>

ClassImp(RMOWaveLengthSimulator)

using namespace std;

static const Double_t kGlambda         = 0.8;          // mean scattaring angle <costheta>
static const Double_t kAlpha           = 0.84;         //
static const Double_t kRefLambda       = (532 * kNANO_METER); // m
static const Double_t kBeffAtRefLambda = (1./24);      // m-1, lambda = kRefLambda;
static const Double_t kA               = (8.0e+39);    // m-1;
static const Double_t kB               = 8300;         // m-1;
static const Double_t kCMdust          = (158 * kBeffAtRefLambda - 0.634);
static const Double_t kKappa           = 1.07;         // m-1;
static const Double_t kFineStrConst    = (1./137);     // fine structure constant;

const Double_t RMOWaveLengthSimulator::kLambdaMin       = (200 * kNANO_METER);
const Double_t RMOWaveLengthSimulator::kLambdaMax       = (800 * kNANO_METER);
const Double_t RMOWaveLengthSimulator::kLambdaMinQE     = (270 * kNANO_METER);
const Double_t RMOWaveLengthSimulator::kLambdaMaxQE     = (720 * kNANO_METER);

//*************************************************************************
// GLOBAL FUNCTIONS
//================================================================
// function CerenkovWLSpectrum
//================================================================

static Double_t CerenkovWaveLengthSpectrum(Double_t *x, Double_t *par)
{

   Double_t norm  = par[0];

   Double_t lambda_um = x[0] / kMICRO_METER;  // [m] to [um]

   Double_t n =   1.55749
                - 1.57988 * lambda_um
                + 3.99993 * lambda_um * lambda_um
                - 4.68271 * lambda_um * lambda_um * lambda_um
                + 2.09354 * lambda_um * lambda_um * lambda_um * lambda_um;

   Double_t beta = 1;                    // for relativistic muon
   Double_t costheta = 1 / (n * beta);
   Double_t sin2theta = (1 + costheta) * (1 - costheta);

   Double_t lambda = x[0];   // [m]

   Double_t dNdL = 2 * TMath::Pi() * kFineStrConst * sin2theta
                   / (lambda * lambda);

   return (norm * dNdL);

}

//================================================================
// function CalcAbsorptionCoefficient
//================================================================

static Double_t CalcAbsorptionCoefficient(Double_t *x, Double_t *par)
{
//
// Returns absorption coefficient at wavelength x[m]
//

   Double_t lambda = x[0] / kNANO_METER;   // [m] to [nm]
   Double_t absorptioncoeff =   kA * TMath::Exp(-0.48175 * lambda)
                              + kB * TMath::Exp(-6700 / lambda)
                              + kCMdust * TMath::Power(lambda, -kKappa); //[m-1]

   return absorptioncoeff;

}


//================================================================
// function GetBeff
//================================================================

static Double_t GetBeff(Double_t lambda)
{
//
// Returns Beff coefficient at wavelength lambda[m]
//

   Double_t ratio = lambda / kRefLambda;
   Double_t beff  = kBeffAtRefLambda * TMath::Power(ratio, - kAlpha); // [m-1]

   return beff;

}

//================================================================
// function WaveLengthSpecturmAtX
//================================================================

static Double_t WaveLengthSpectrumAtX(Double_t *x, Double_t *par)
{
   Double_t cwlpar[1];    // Cerenkov parameter array
   
   Double_t lambda = x[0]; 
   Double_t dist   = par[0];
   cwlpar[0]       = par[1];    // normalization of Cerenkov spectrum
   Double_t norm   = par[2];    // normalization of this function 
   Double_t dum[1];

   Double_t origflax = CerenkovWaveLengthSpectrum(x, cwlpar);
   Double_t scateff  = GetBeff(lambda);
   Double_t abseff   = CalcAbsorptionCoefficient(x, dum); 

   Double_t leffinv  = TMath::Sqrt(scateff * abseff);
   return (norm * origflax * TMath::Exp(- dist * leffinv));

}


//=====================================================================
//* constructor -------------------------------------------------------

RMOWaveLengthSimulator::RMOWaveLengthSimulator()
                   : RMOVModule("RMOWaveLengthSimulator","Cerenkov WaveLength Simulator"),
                     fCerenkovWaveLengthSpectrum(0), fWaveLengthSpectrumAtX(0),
                     fWaveLengthHist(0), fOutputPhotonBuf(0), fCanvas(0)
{
}

//=====================================================================
//* destructor --------------------------------------------------------

RMOWaveLengthSimulator::~RMOWaveLengthSimulator()
{
//*-*-*-*-*-*-*-*-*-*-*-*Destructor*-*-*-*-*-*-*-*-*-*-*-*-*
//                       ==========
   if (fCerenkovWaveLengthSpectrum) delete fCerenkovWaveLengthSpectrum;
   if (fWaveLengthSpectrumAtX)      delete fWaveLengthSpectrumAtX;
   if (fWaveLengthHist)             delete fWaveLengthHist;
   if (fOutputPhotonBuf)            delete fOutputPhotonBuf;
   if (fCanvas)                     delete fCanvas;
}

//===================================================================
//* Configure ---------------------------------------------------------

void RMOWaveLengthSimulator::Configure()
{

//*-*-*-*-*-*-*-*-*-*-*-*Configure*-*-*-*-*-*-*-*-*-*-*-*-*
//                       ==========
//     

   Int_t npx = (Int_t) ((kLambdaMax - kLambdaMin) / kNANO_METER + 1);
   fCerenkovWaveLengthSpectrum = new TF1("CerenkovWaveLengthSpectrum", 
                                          CerenkovWaveLengthSpectrum,
                                          kLambdaMin, kLambdaMax, 1);
   fCerenkovWaveLengthSpectrum->SetParameter(0, 1.);
   fCerenkovWaveLengthSpectrum->SetNpx(npx);
   fCerenkovWaveLengthSpectrum->SetLineColor(kRed);

   fWaveLengthSpectrumAtX = new TF1("WaveLengthSpectrumAtX", WaveLengthSpectrumAtX, 
                                     kLambdaMin, kLambdaMax, 3);
   fWaveLengthSpectrumAtX->SetNpx(npx);
   fWaveLengthSpectrumAtX->SetLineColor(kCyan);

}

//=====================================================================
//* Process -----------------------------------------------------------

void RMOWaveLengthSimulator::Process()
{
//*-*-*-*-*-*-*-*-*-*-*-*Process*-*-*-*-*-*-*-*-*-*-*-*-*
//                       =======

   /* Get Input Data --------------------- */

   RMOMCPhotonHitBuf imputbuf = (RMOMCPhotonHitBuf &)GetImputBuffer();
   TIter             iter  = imputbuf.GetIter();

   /* Convert pe to charge --------------- */
   if (fOutputPhotonBuf) delete fOutputPhotonBuf;

   fOutputPhotonBuf = new RMOPhotonBuf();
   fOutputPhotonBuf->SetOwner();

   RMOMCPhotonHit *hit;
   while ((Int_t)(hit = (RMOMCPhotonHit *)iter.Next())) {
      
      RMOOpticalPhoton *oph = new RMOOpticalPhoton();
      Double_t time = hit->GetTime();
      oph->SetTime(time);
      TVector3 direction = hit->GetDirection();
      oph->SetDirection(direction);

      Double_t distance  = hit->GetDistanceFromSource();
      fWaveLengthSpectrumAtX->SetParameter(0, distance); // distance from source
      fWaveLengthSpectrumAtX->SetParameter(1, 1.); // normalizeaton of Cerenkov spectrum
      fWaveLengthSpectrumAtX->SetParameter(2, 1.); // normalization

      Double_t wavelen = fWaveLengthSpectrumAtX->GetRandom(kLambdaMinQE, kLambdaMaxQE);
      oph->SetWaveLength(wavelen);

      oph->SetMother(hit);
      fOutputPhotonBuf->Add(oph);
   }
   
}

//=====================================================================
//* Finish ------------------------------------------------------------

void RMOWaveLengthSimulator::Finish()
{


}

//=====================================================================
//* Draw --------------------------------------------------------------

void RMOWaveLengthSimulator::Draw()
{
   if (fCanvas)          delete fCanvas;
   if (fWaveLengthHist)  delete fWaveLengthHist;

   Int_t nbins = (Int_t)((kLambdaMax - kLambdaMin) / kNANO_METER + 1);
   fWaveLengthHist = new TH1D("WaveLengthSpectrum", "WaveLengthSpectrum",
                               nbins, kLambdaMin, kLambdaMax);
   
   TIter iter = fOutputPhotonBuf->GetIter();
   RMOOpticalPhoton *datum;
   while ((Int_t)(datum = (RMOOpticalPhoton *)iter.Next())) {
      Double_t lambda = datum->GetWaveLength();
      fWaveLengthHist->Fill(lambda);
   }
      
   fCanvas  = new TCanvas("WaveLengthSpectrum", "WaveLengthSpectrum",
                          10, 10, 1200, 400);
   fCanvas->Divide(3, 1);
   fCanvas->cd(1); fCerenkovWaveLengthSpectrum->Draw();
   fCanvas->cd(2); fWaveLengthSpectrumAtX->Draw();
   fCanvas->cd(3); fWaveLengthHist->Draw();
}



