// $Id: RMOOpticalPhoton.cxx,v 1.5 2004/09/26 03:42:41 hoshina Exp $
//*************************************************************************
//* --------------------
//* RMOOpticalPhoton
//* --------------------
//* (Description)
//*	Particle class which represents an optical photon.
//* (Update Record)
//*	2003/12/22  K.Hoshina	Original version.
//*     2004/8/23   S.Yoshida   Adding the wavelength/ 
//*                             the cathode coordinate field
//*************************************************************************
#include <TMath.h>

#include "RMOOpticalPhoton.h"
#include "RMOUnit.h"

ClassImp(RMOOpticalPhoton)

using namespace std;

const  TString  RMOOpticalPhoton::kParticleName    = "OpticalPhoton";
const  Double_t RMOOpticalPhoton::kCharge          = 0;
const  Double_t RMOOpticalPhoton::kMaxWaveLength   = 1.0e3*kNANO_METER; // 1000 nm
const  Double_t RMOOpticalPhoton::kMinWaveLength   = 1.0e2*kNANO_METER; // 100 nm


//=====================================================================
//* constructor -------------------------------------------------------

RMOOpticalPhoton::RMOOpticalPhoton()
                 :RMOVParticle(), RMOVAttDatum()
{
  fWaveLength = kMinWaveLength;
  fCosTheta = 1.0;
  fAzimuthAngle = 0.0;
}

RMOOpticalPhoton::RMOOpticalPhoton(const TVector3 &pos, const TVector3 &dir)
                 :RMOVParticle(pos, dir), RMOVAttDatum()
{
  fWaveLength = kMinWaveLength;
  fCosTheta = 1.0;
  fAzimuthAngle = 0.0;
}

//=====================================================================
//* destructor --------------------------------------------------------

RMOOpticalPhoton::~RMOOpticalPhoton()
{
}

//=====================================================================
//* Check the waveLength range ----------------------------------------

Bool_t RMOOpticalPhoton::IsValidWaveLength(Double_t lambda) const
{
  if(kMinWaveLength <= lambda && lambda <= kMaxWaveLength) return true;
  return false;
}

//=====================================================================
//* WaveLength Set ----------------------------------------------------

void RMOOpticalPhoton::SetWaveLength(Double_t lambda)
{
  if(IsValidWaveLength(lambda)){
    fWaveLength = lambda;
    Double_t energy = 1240. / (lambda / kNANO_METER);  // eV 
    SetEnergy(energy);
    //cerr << "Set energy [eV] " << energy << endl;
  }else{
    cerr << "Invalid wavelength (" << lambda << ") [m]" << endl;
    cerr << "Wavelength unchanged (" << fWaveLength << ") [m]" << endl;
  }
}

//=====================================================================
//* Setting the cosine  -----------------------------------------------

void RMOOpticalPhoton::SetCosTheta(Double_t cosTheta)
{
  if(TMath::Abs(cosTheta)<=1.0){
    fCosTheta = cosTheta;
  }else{
    cerr << "Invalid cosine (" << cosTheta << ")" << endl;
    cerr << "Cosine unchanged (" << fCosTheta << ")" << endl;
  }
}

//=====================================================================
//* Setting the Azimuth Angle  ----------------------------------------
//* alpha: Azimuth Angle [rad] ----------------------------------------

void RMOOpticalPhoton::SetAzimuth(Double_t alpha)
{
  if(0.0<= alpha && alpha<=2.0*TMath::Pi()){
    fAzimuthAngle = alpha;
  }else{
    cerr << "Invalid azimuth (" << alpha*TMath::RadToDeg() 
	 << ") [deg]"  << endl;
    cerr << "Cosine unchanged (" << fAzimuthAngle*TMath::RadToDeg()
	 << ") [deg]"  << endl;
  }
}
