//*************************************************************************
//* ---------------------
//* Interpolation Class
//* ---------------------
//* (Description)
//*     Interpolating data points based on the Numerical Recipes.
//*     Ported originally from the JULIeT numRecipes package written by Java
//* (Requires)
//* (Provides)
//*	Interpolation              : static methods for interpolation
//* (Update Record)
//*	2004/8/13(FRIday!!)  S.Yoshida	C++ version.
//*************************************************************************
#include "Interpolation.h"

Double_t Interpolation::fDelta = 1.0;

ClassImp(Interpolation)

using namespace std;


Int_t Interpolation::SearchIndex(const TArrayD& xx, const Double_t x, 
					const Int_t n)
{
//******************************************************************
// Search the array index such that x is between xx[index]
// and xx[index+1]. xx must be monotonic, either increasing
// or decreasing. index =0 or index = n-1 is returned
// to indicate x is out of range.  n gives number of the elements
// i.e. xx[0....n-1].
//******************************************************************
  Int_t jm;
  Int_t jl=0;
  Int_t ju = n;
  Int_t index;
  Bool_t ascnd;

  ascnd=(xx[n-1] >= xx[0]);
  while (ju-jl > 1) {
    jm=(ju+jl) >> 1;
    if ((x >= xx[jm]) == ascnd)
      jl=jm;
    else
      ju=jm;
  }
  if (x == xx[0]) index=0;
  else if(x == xx[n-1]) index=n-1;
  else index=jl;

  return index;
}




Double_t Interpolation::PolynominalInterpolate(const TArrayD& xa, 
					       const TArrayD& ya, 
					       const Int_t n, 
					       Double_t x)
{
//******************************************************************
// Given arrays xa[0....n-1] and ya[0....n-1], and given a value x,
// this method returns a value y by the polynominal interpolation.
// Number of data ponts n is qeual to the plynominal of degree. 
//******************************************************************
  Int_t dim = xa.fN;
  if(dim>n) dim = n;
  TArrayD* c = new TArrayD(ya);
  TArrayD* d = new TArrayD(ya);

  Double_t dif = TMath::Abs(x-xa[0]);
  Double_t difNew;
  Int_t closestIndex = 0;

  for(Int_t i=0;i<dim;i++){ // Here we find the index of the closest table.
    difNew = TMath::Abs(x-xa[i]);
    if(difNew< dif){
      closestIndex = i;
      dif = difNew;
    }
  }

  Double_t yOut = ya[closestIndex--];
  for(Int_t m=1;m<dim;m++){
    for(Int_t i=0;i<dim-m;i++){
      Double_t ho = xa[i]-x;
      Double_t hp = xa[i+m]-x;
      Double_t w = c->At(i+1)- d->At(i);
      Double_t den = ho - hp;
      if(den == 0.0){
	cerr << "Error in polynominalInterpolate" << endl;
	exit(0);
      }
      den = w/den;
      c->AddAt(ho*den,i);
      d->AddAt(hp*den,i);
    }

    fDelta = ((2.0*closestIndex < (dim-m)) ? c->At(closestIndex+1) : 
	     d->At(closestIndex--));
    yOut += fDelta;
  }

  delete c;
  delete d;

  return yOut;
}



Double_t Interpolation::GetErrorInPolynominalInterpolate( ) {
  return fDelta;
}


Double_t Interpolation::MThPolynominalInterpolate(const TArrayD& xa, 
							 const TArrayD& ya, 
							 Int_t n,
							 Double_t x, Int_t m)
{
//******************************************************************
// Interpolate with mTh pylinominal function. The data arrays
// xa[0.....n-1] and ya[0....n-1] must be larger than m, 
// polynominal of degree. 
//******************************************************************

  Int_t index = SearchIndex(xa, x, n);
  Int_t kMax = ((index-(m-1)/2 > 1) ? index-(m-1)/2 : 1);
  Int_t kMin = ((kMax < n+1-m) ? kMax : n+1-m);

  TArrayD* xpart = new TArrayD(m);
  TArrayD* ypart = new TArrayD(m);

  for(Int_t i=0; i<m; i++){
    xpart->AddAt(xa[kMin-1+i],i);
    ypart->AddAt(ya[kMin-1+i],i);
  }


  Double_t yOut = PolynominalInterpolate(*xpart,*ypart,m,x);

  delete xpart;
  delete ypart;

  return yOut;

}
