//*************************************************************************
//* ---------------------
//* RMOPMT2DCollectEff Class
//* ---------------------
//* (Description)
//*     module class for PMT Qe
//* (Requires)
//* (Provides)
//*	RMOPMT2DCollectEff              : 2D CollectionEfficiency modules
//* (Update Record)
//*	2004/8/16  S.Yoshida	Original version.
//*************************************************************************
#include <TMath.h>

#include "RMOPMT2DCollectEff.h"
#include "RMOUnit.h"
#include "Interpolation.h"

const Int_t    RMOPMT2DCollectEff::kNXbins = 30;
const Int_t    RMOPMT2DCollectEff::kNYbins = 30;
const Double_t RMOPMT2DCollectEff::kXmin   = (-15. * kCENTI_METER);
const Double_t RMOPMT2DCollectEff::kXmax   = (15. * kCENTI_METER);
const Double_t RMOPMT2DCollectEff::kYmin   = (-15. * kCENTI_METER);
const Double_t RMOPMT2DCollectEff::kYmax   = (15. * kCENTI_METER);

const Double_t RMOPMT2DCollectEff::kR_pmt = (13.669 * kCENTI_METER); // 13.669 [cm]
const  Int_t RMOPMT2DCollectEff::kDegOfPolynominal = 6;

ClassImp(RMOPMT2DCollectEff)

using namespace std;

//=====================================================================
//* constructor -------------------------------------------------------

RMOPMT2DCollectEff::RMOPMT2DCollectEff()
  : RMOVDatum(), fCeDataName("sf0050_1800V.table"), fCETable(0), 
    fDistanceTable(0), fAzimuthTable(0), fNumberTable(0),
    fCEMap(0), fCanvas(0)
{
   fScale = 1.0; fNorm = 1.0;
   ReadCETable();
}

RMOPMT2DCollectEff::RMOPMT2DCollectEff(TString &fileName)
  : RMOVDatum(), fCETable(0), 
    fDistanceTable(0), fAzimuthTable(0), fNumberTable(0),
    fCEMap(0), fCanvas(0)
{
   fCeDataName = fileName;
   fScale = 1.0; fNorm = 1.0;
   ReadCETable();
}

//=====================================================================
//* destructor --------------------------------------------------------

RMOPMT2DCollectEff::~RMOPMT2DCollectEff()
{
   if (fCETable){
     TArrayD* ceTablePtr;
     ceTablePtr = fCETable;
     for(Int_t i=0;i<kNumber_of_azimuth;i++){
       delete ceTablePtr;
       ceTablePtr++;
     }
     delete[] fCETable;
   }

   if (fDistanceTable){
     TArrayD* distanceTablePtr;
     distanceTablePtr = fDistanceTable;
     for(Int_t i=0;i<kNumber_of_azimuth;i++){
       delete distanceTablePtr;
       distanceTablePtr++;
     }
     delete[] fDistanceTable;
   }

   if (fAzimuthTable)  delete fAzimuthTable;
   if (fNumberTable)   delete fNumberTable;
   if (fCEMap)         delete fCEMap;
   if (fCanvas)        delete fCanvas;

}
//=====================================================================
//* Draw --------------------------------------------------------------

void RMOPMT2DCollectEff::Draw()
{
   if (fCanvas) delete fCanvas;
   if (fCEMap)  delete fCEMap;

   fCanvas = new TCanvas("Uniformity_CEMap", "Uniformity of Collection Efficiency", 
                          10, 10, 800, 400);

   fCEMap  = new TH2D("CollectionEfficiencyMap", "CollectionEfficiencyMap",
                               kNXbins, kXmin, kXmax, 
                               kNYbins, kYmin, kYmax );

   Double_t x,y;
   Double_t epsilon = 0.1 * kCENTI_METER; // Adjustment of the binning effect

   for (Int_t ix = 0; ix < kNXbins; ix++) {
     x = kXmin + epsilon + ((Double_t)ix) * kCENTI_METER;   //[cm]
     for (Int_t iy = 0; iy<kNYbins; iy++) {
       y = kYmin + epsilon + ((Double_t)iy) * kCENTI_METER; //[nm]
       Double_t d = TMath::Sqrt( x*x + y*y );
       Double_t alpha = 0.0;
       if (x > 0.0) alpha = TMath::ATan( y / x );
       if (x < 0.0) alpha = TMath::ATan( y / x ) + TMath::Pi();
       if (x == 0.0) {
	 if (y >= 0.0) alpha = TMath::Pi() / 2.0;
	 else alpha = 3.0 * TMath::Pi() / 2.0;
       }
       if (alpha < 0.0) alpha += 2.0 * TMath::Pi();

       Double_t theta = d / kR_pmt;
       Double_t cosTheta = TMath::Cos(theta);

       Double_t pmtCE = Get2DCollectEff(alpha, cosTheta);

       Int_t    bin = fCEMap->FindBin(x, y);
       Double_t content = fCEMap->GetBinContent(bin);

       if (content) {
	 pmtCE = 0.5 * (content + pmtCE); 
	 fCEMap->SetBinContent(bin, pmtCE);
       } else {
	 fCEMap->Fill(x, y, pmtCE);
       }
     }
   }

   fCEMap->GetXaxis()->SetTitle(" X [m]");
   fCEMap->GetYaxis()->SetTitle(" Y [m]");
   fCanvas->cd();
   fCEMap->Draw("LEGO2");

}

void RMOPMT2DCollectEff::ReadCETable()
{
//=====================================================================
// Read the 2 Dimensional Collection Efficiency Table -----------------
//=====================================================================
   // uniformity data open

   ifstream in(const_cast<Char_t*>(fCeDataName.Data()));

   if (!in.good()) {
      cerr << "RMOPMT2DCollectEff::ReadCETable: file " << fCeDataName
           << " open failed! abort." << endl;
      abort();
   } 

   // Table Memory Allocation
   fAzimuthTable = new TArrayD(kNumber_of_azimuth);
   fNumberTable = new TArrayI(kNumber_of_azimuth);

   fDistanceTable = new TArrayD[kNumber_of_azimuth](kNumber_of_distance);
   TArrayD* distanceTablePtr;

   fCETable = new TArrayD[kNumber_of_azimuth](kNumber_of_distance);
   TArrayD* ceTablePtr;


   // Reading the table

   in >> fPMTName;
   cerr << "PMT name " << fPMTName << endl;

   Int_t n,k;
   Double_t  azimuth, distance, pmtCE, pmtCEError;

   ceTablePtr = fCETable;
   distanceTablePtr = fDistanceTable;
   for (n=0; n<kNumber_of_azimuth; n++){

     Int_t j = 0;
     for (k=0; k<(kNumber_of_distance-1); k++) {

       in >> azimuth >> distance >> pmtCE >> pmtCEError;

       fAzimuthTable->AddAt(azimuth * TMath::DegToRad(), n); //[rad]

       if (pmtCE > 0.0) { // When Efficiency number makes sense
	 ceTablePtr->AddAt(pmtCE,j);
	 distanceTablePtr->AddAt(distance * kCENTI_METER, j); //[cm]
	 j++;
       }
     }
     fNumberTable->AddAt(j+1,n);
     distance = distanceTablePtr->At(j-1) + 0.15*kCENTI_METER;
     distanceTablePtr->AddAt(distance,j);
     pmtCE = ceTablePtr->At(j-1);
     ceTablePtr->AddAt(pmtCE,j);

     cerr << "Azimuth = " << fAzimuthTable->At(n)*TMath::RadToDeg() 
	  << "[deg]  # of Data (" << fNumberTable->At(n)-1
	  << ")  EffAtCenter = " << ceTablePtr->At(0) << "[%]" << endl;

     ceTablePtr++;
     distanceTablePtr++;

   }

//   ceTablePtr = fCETable;
//   distanceTablePtr = fDistanceTable;
//   for(Int_t i=0;i<fNumberTable->At(0);i++){
//     cerr << "Distance = " << distanceTablePtr->At(i) << "[m] Eff = "
//	  << ceTablePtr->At(i) << "[%]" << endl;
//   }

   NormalizeCollectEff();

   in.close();
   in.clear();


}

void RMOPMT2DCollectEff::NormalizeCollectEff()
{
//=====================================================================
// Normalize the 2D collection efficiency avraging over solid angle ---
//  100 % is assumed for the overall averaged efficiency            ---
//=====================================================================

  Int_t n_bin = 100;
  Double_t deltaCosine = 1.0/(Double_t )n_bin;
  Double_t deltaAlpha = 2.0*TMath::Pi()/(Double_t )n_bin;

  Int_t count = 0;
  Double_t sum = 0.0;
  for(Int_t icos =0; icos<n_bin; icos++){
    Double_t cosTheta = deltaCosine*(Double_t )icos;
    for(Int_t ialpha =0; ialpha<n_bin; ialpha++){
      Double_t alpha = deltaAlpha*(Double_t )ialpha;
      Double_t pmt2DCE = Get2DCollectEff(alpha, cosTheta);
      sum += pmt2DCE;
      if(pmt2DCE>0.0) count ++;
    }
  }

  fNorm = 100.0/(sum/(Double_t)count);
  //  cerr << "Normalization " << fNorm << endl;
}



Double_t RMOPMT2DCollectEff::CosZenithToDistance(Double_t cos_th)
{
//=====================================================================
// Convert to distance along the R-guid from cosine of Zenith----------
//=====================================================================
  Double_t theta = TMath::ACos(cos_th);
  Double_t d = theta*kR_pmt;

  return(d);
}

Double_t RMOPMT2DCollectEff::GetDistanceMax() const
{
//=====================================================================
// Maximimum Distance allowed by the data point
//=====================================================================
  if(fNumberTable){
    Int_t n = fNumberTable->At(0) - 2;

    TArrayD* distanceTablePtr;
    distanceTablePtr = fDistanceTable;

    return distanceTablePtr->At(n);
  }else{
    return 0.0;
  }
}

Bool_t RMOPMT2DCollectEff::IsValidDistance(Double_t l) const
{
//=====================================================================
// Check if the given distance is in the allowed range
//=====================================================================
  if(fAzimuthTable){
    if(GetDistanceMin() <= l &&  l <= GetDistanceMax()) return kTRUE;
    else return kFALSE;
  }else{
    cerr << "CE table has not been read out, yet!" << endl;
    abort();
  }
 
}
  

Double_t RMOPMT2DCollectEff::Get2DCollectEff(Double_t azimuthAngle, 
					Double_t cosZenith)
{
//=====================================================================
// Get 2D Collection Efficieny [%]-------------------------------------
// azimuthAngle: Azimuth Angle on the PMT photocathod [rad]
// cosZenith   : Cosine of Zenith Angle on the PMT photocathod
//=====================================================================

  Double_t l = CosZenithToDistance(cosZenith);
  if(!IsValidDistance(l)){
    if(l>GetDistanceMax()) return 0.0;
    else if (0<= l && l < GetDistanceMin()) l = GetDistanceMin();
    else return 0.0;
  }

  Int_t index = Interpolation::SearchIndex(*fAzimuthTable,azimuthAngle,kNumber_of_azimuth);
  if(index >= (kNumber_of_azimuth-1)) index = kNumber_of_azimuth-2;
  Double_t azimuthAngleLow = fAzimuthTable->At(index);
  Double_t azimuthAngleUp = fAzimuthTable->At(index+1);

  TArrayD* distanceTablePtr;
  TArrayD* ceTablePtr;

  Double_t pmtCELow;
  distanceTablePtr = fDistanceTable;
  distanceTablePtr += index;
  ceTablePtr = fCETable;
  ceTablePtr += index;
  Int_t index_distance = 
    Interpolation::SearchIndex(*distanceTablePtr,l,fNumberTable->At(index));

  if(TMath::Abs(distanceTablePtr->At(index_distance+1)-
		distanceTablePtr->At(index_distance))
     >1.0*kCENTI_METER){ // If Large interval
    pmtCELow = ceTablePtr->At(index_distance)+
      (l-distanceTablePtr->At(index_distance))*
      (ceTablePtr->At(index_distance+1)-ceTablePtr->At(index_distance))/
      (distanceTablePtr->At(index_distance+1)-distanceTablePtr->At(index_distance));
  }else{
    pmtCELow = 
      Interpolation::MThPolynominalInterpolate(*distanceTablePtr,
	    *ceTablePtr,fNumberTable->At(index),l,kDegOfPolynominal);
  }

  Double_t pmtCEUp;
  distanceTablePtr++;
  ceTablePtr++;
  index_distance = 
    Interpolation::SearchIndex(*distanceTablePtr,l,fNumberTable->At(index+1));

  if(TMath::Abs(distanceTablePtr->At(index_distance+1)-
		distanceTablePtr->At(index_distance))
     >1.0*kCENTI_METER){ // If Large interval
    pmtCEUp = ceTablePtr->At(index_distance)+
      (l-distanceTablePtr->At(index_distance))*
      (ceTablePtr->At(index_distance+1)-ceTablePtr->At(index_distance))/
      (distanceTablePtr->At(index_distance+1)-distanceTablePtr->At(index_distance));
  }else{
    pmtCEUp = 
      Interpolation::MThPolynominalInterpolate(*distanceTablePtr,
	    *ceTablePtr,fNumberTable->At(index+1),l,kDegOfPolynominal);
  }

  Double_t azimuthAngleRatio = 
    (azimuthAngle-azimuthAngleLow)/(azimuthAngleUp-azimuthAngleLow);
  Double_t pmtCE = pmtCELow + (pmtCEUp-pmtCELow)*azimuthAngleRatio;

  return pmtCE*fScale*fNorm;

}

