//*************************************************************************
//* ---------------------
//* RMOPMT2DGain Class
//* ---------------------
//* (Description)
//*     module class for PMT Gain
//* (Requires)
//* (Provides)
//*	RMOPMT2DGain              : 2D Gain modules
//* (Update Record)
//*	2004/8/25  S.Yoshida	Original version.
//*************************************************************************
#include <TMath.h>

#include "RMOPMT2DGain.h"
#include "RMOUnit.h"
#include "Interpolation.h"

const Int_t    RMOPMT2DGain::kNXbins = 30;
const Int_t    RMOPMT2DGain::kNYbins = 30;
const Double_t RMOPMT2DGain::kXmin   = (-15. * kCENTI_METER);
const Double_t RMOPMT2DGain::kXmax   = (15. * kCENTI_METER);
const Double_t RMOPMT2DGain::kYmin   = (-15. * kCENTI_METER);
const Double_t RMOPMT2DGain::kYmax   = (15. * kCENTI_METER);

const Double_t RMOPMT2DGain::kR_pmt = (13.669 * kCENTI_METER);
// 13.669 [cm]
const  Int_t RMOPMT2DGain::kDegOfPolynominal = 6;

ClassImp(RMOPMT2DGain)

using namespace std;

//=====================================================================
//* constructor -------------------------------------------------------

RMOPMT2DGain::RMOPMT2DGain(TString &fileName)
  : RMOVDatum(), fGainMap(0),fGainTable(0), 
    fDistanceTable(0), fAzimuthTable(0), fNumberTable(0),
    fCanvas(0)
{
   fGainDataName = fileName;
   fScale = 1.0; fNorm = 1.0;
   ReadGainTable();
}

//=====================================================================
//* destructor --------------------------------------------------------

RMOPMT2DGain::~RMOPMT2DGain()
{
   if (fGainTable){
     TArrayD* ceTablePtr;
     ceTablePtr = fGainTable;
     for(Int_t i=0;i<kNumber_of_azimuth;i++){
       delete ceTablePtr;
       ceTablePtr++;
     }
     delete[] fGainTable;
   }

   if (fDistanceTable){
     TArrayD* distanceTablePtr;
     distanceTablePtr = fDistanceTable;
     for(Int_t i=0;i<kNumber_of_azimuth;i++){
       delete distanceTablePtr;
       distanceTablePtr++;
     }
     delete[] fDistanceTable;
   }

   if (fAzimuthTable)  delete fAzimuthTable;
   if (fNumberTable)   delete fNumberTable;
   if (fGainMap)         delete fGainMap;
   if (fCanvas)        delete fCanvas;

}
//=====================================================================
//* Draw --------------------------------------------------------------

void RMOPMT2DGain::Draw()
{
   if (fCanvas) delete fCanvas;
   if (fGainMap)  delete fGainMap;

   fCanvas = new TCanvas("Uniformity_GainMap", "2 Dimensional Gain", 
                          10, 10, 800, 400);

   fGainMap  = new TH2D("2DGainMap", "2DGainMap",
                               kNXbins, kXmin, kXmax, 
                               kNYbins, kYmin, kYmax );

   Double_t x,y;
   Double_t epsilon = 0.1*kCENTI_METER; // Adjustment of the binning effect

   for(Int_t ix = 0;ix<kNXbins; ix++){
     x = kXmin + epsilon + ((Double_t)ix)*kCENTI_METER;//[cm]
     for(Int_t iy = 0;iy<kNYbins; iy++){
       y = kYmin + epsilon + ((Double_t)iy)*kCENTI_METER;//[nm]
       Double_t d = TMath::Sqrt(x*x+y*y);
       Double_t alpha = 0.0;
       if(x>0.0) alpha = TMath::ATan(y/x);
       if(x<0.0) alpha = TMath::ATan(y/x)+ TMath::Pi();
       if(x == 0.0){
	 if(y>=0.0) alpha = TMath::Pi()/2.0;
	 else alpha = 3.0*TMath::Pi()/2.0;
       }
       if(alpha<0.0) alpha += 2.0*TMath::Pi();

       Double_t theta = d/kR_pmt;
       Double_t cosTheta = TMath::Cos(theta);

       Double_t pmtGain = Get2DGain(alpha, cosTheta);

       Int_t    bin = fGainMap->FindBin(x, y);
       Double_t content = fGainMap->GetBinContent(bin);

       if (content) {
	 pmtGain = 0.5 * (content + pmtGain); 
	 fGainMap->SetBinContent(bin, pmtGain);
       } else {
	 fGainMap->Fill(x, y, pmtGain);
       }
     }
   }

   fGainMap->GetXaxis()->SetTitle(" X [m]");
   fGainMap->GetYaxis()->SetTitle(" Y [m]");
   fCanvas->cd();
   fGainMap->Draw("LEGO2");

}
//=====================================================================
//* Draw --------------------------------------------------------------

void RMOPMT2DGain::MakeMapWithoutDraw()
{
   if (fGainMap)  delete fGainMap;

   fGainMap  = new TH2D("2DGainMap", "2DGainMap",
                               kNXbins, kXmin, kXmax, 
                               kNYbins, kYmin, kYmax );

   Double_t x,y;
   Double_t epsilon = 0.1*kCENTI_METER; // Adjustment of the binning effect

   for(Int_t ix = 0;ix<kNXbins; ix++){
     x = kXmin + epsilon + ((Double_t)ix)*kCENTI_METER;//[cm]
     for(Int_t iy = 0;iy<kNYbins; iy++){
       y = kYmin + epsilon + ((Double_t)iy)*kCENTI_METER;//[nm]
       Double_t d = TMath::Sqrt(x*x+y*y);
       Double_t alpha = 0.0;
       if(x>0.0) alpha = TMath::ATan(y/x);
       if(x<0.0) alpha = TMath::ATan(y/x)+ TMath::Pi();
       if(x == 0.0){
	 if(y>=0.0) alpha = TMath::Pi()/2.0;
	 else alpha = 3.0*TMath::Pi()/2.0;
       }
       if(alpha<0.0) alpha += 2.0*TMath::Pi();

       Double_t theta = d/kR_pmt;
       Double_t cosTheta = TMath::Cos(theta);

       Double_t pmtGain = Get2DGain(alpha, cosTheta);

       Int_t    bin = fGainMap->FindBin(x, y);
       Double_t content = fGainMap->GetBinContent(bin);

       if (content) {
	 pmtGain = 0.5 * (content + pmtGain); 
	 fGainMap->SetBinContent(bin, pmtGain);
       } else {
	 fGainMap->Fill(x, y, pmtGain);
       }
     }
   }

}

//=====================================================================
// Read the 2 Dimensional Collection Efficiency Table -----------------
//=====================================================================
void RMOPMT2DGain::ReadGainTable()
{
   // uniformity data open

   ifstream in(const_cast<Char_t*>(fGainDataName.Data()));

   if (!in.good()) {
      cerr << "RMOPMT2DGain::ReadGainTable: file " << fGainDataName
           << " open failed! abort." << endl;
      abort();
   } 

   // Table Memory Allocation
   fAzimuthTable = new TArrayD(kNumber_of_azimuth);
   fNumberTable = new TArrayI(kNumber_of_azimuth);

   fDistanceTable = new TArrayD[kNumber_of_azimuth](kNumber_of_distance);
   TArrayD* distanceTablePtr;

   fGainTable = new TArrayD[kNumber_of_azimuth](kNumber_of_distance);
   TArrayD* ceTablePtr;


   // Reading the table

   in >> fPMTName;
   cerr << "PMT name " << fPMTName << endl;

   Int_t n,k;
   Double_t  azimuth, distance, pmtGain, pmtGainError;

   ceTablePtr = fGainTable;
   distanceTablePtr = fDistanceTable;
   for(n=0;n<kNumber_of_azimuth;n++){

     Int_t j = 0;
     for(k=0;k<(kNumber_of_distance-1);k++){

       in >> azimuth >> distance >> pmtGain >> pmtGainError;

       fAzimuthTable->AddAt(azimuth*TMath::DegToRad(),n); //[rad]

       if(pmtGain>0.0){ // When Efficiency number makes sense
	 ceTablePtr->AddAt(pmtGain,j);
	 distanceTablePtr->AddAt(distance*kCENTI_METER,j); //[cm]
	 j++;
       }
     }
     fNumberTable->AddAt(j+1,n);
     distance = distanceTablePtr->At(j-1) + 0.15*kCENTI_METER;
     distanceTablePtr->AddAt(distance,j);
     pmtGain = ceTablePtr->At(j-1);
     ceTablePtr->AddAt(pmtGain,j);

     ceTablePtr++;
     distanceTablePtr++;

   }

//   ceTablePtr = fGainTable;
//   distanceTablePtr = fDistanceTable;
//   for(Int_t i=0;i<fNumberTable->At(0);i++){
//     cerr << "Distance = " << distanceTablePtr->At(i) << "[m] Gain = "
//	  << ceTablePtr->At(i) << endl;
//   }

   NormalizeGain();

   in.close();
   in.clear();


}

//=====================================================================
// Normalize the 2D collection efficiency avraging over solid angle ---
//  100 % is assumed for the overall averaged efficiency            ---
//=====================================================================
void RMOPMT2DGain::NormalizeGain()
{

  Int_t n_bin = 100;
  Double_t deltaCosine = 1.0/(Double_t )n_bin;
  Double_t deltaAlpha = 2.0*TMath::Pi()/(Double_t )n_bin;

  Int_t count = 0;
  Double_t sum = 0.0;
  Double_t pmt2DGain;
  for(Int_t icos =0; icos<n_bin; icos++){
    Double_t cosTheta = deltaCosine*(Double_t )icos;
    for(Int_t ialpha =0; ialpha<n_bin; ialpha++){
      Double_t alpha = deltaAlpha*(Double_t )ialpha;
      pmt2DGain = Get2DGain(alpha, cosTheta);
      sum += pmt2DGain;
      if(pmt2DGain>0.0) count ++;
    }
    Double_t l = CosZenithToDistance(cosTheta);
  }

  fNorm = 1.0/(sum/(Double_t)count);
  //  cerr << "Normalization " << fNorm << endl;

}



//=====================================================================
// Convert to distance along the R-guid from cosine of Zenith----------
//=====================================================================
Double_t RMOPMT2DGain::CosZenithToDistance(Double_t cos_th)
{
  Double_t theta = TMath::ACos(cos_th);
  Double_t d = theta*kR_pmt;

  return(d);
}

//=====================================================================
// Maximimum Distance allowed by the data point
//=====================================================================
Double_t RMOPMT2DGain::GetDistanceMax() const
{
  if(fNumberTable){
    Int_t n = fNumberTable->At(0) - 2;

    TArrayD* distanceTablePtr;
    distanceTablePtr = fDistanceTable;

    return distanceTablePtr->At(n);
  }else{
    return 0.0;
  }
}

//=====================================================================
// Check if the given distance is in the allowed range
//=====================================================================
Bool_t RMOPMT2DGain::IsValidDistance(Double_t l) const
{
  if(fAzimuthTable){
    if(GetDistanceMin() <= l &&  l <= GetDistanceMax()) return kTRUE;
    else return kFALSE;
  }else{
    cerr << "Gain table has not been read out, yet!" << endl;
    abort();
  }
 
}
  

//=====================================================================
// Get 2D Collection Efficieny [%]-------------------------------------
// azimuthAngle: Azimuth Angle on the PMT photocathod [rad]
// cosZenith   : Cosine of Zenith Angle on the PMT photocathod
//=====================================================================
Double_t RMOPMT2DGain::Get2DGain(Double_t azimuthAngle, 
					Double_t cosZenith)
{

  Double_t l = CosZenithToDistance(cosZenith);
  if(!IsValidDistance(l)){
    if(l>GetDistanceMax()) return 0.0;
    else if (0<= l && l < GetDistanceMin()) l = GetDistanceMin();
    else return 0.0;
  }

  Int_t index = Interpolation::SearchIndex(*fAzimuthTable,azimuthAngle,kNumber_of_azimuth);
  if(index >= (kNumber_of_azimuth-1)) index = kNumber_of_azimuth-2;
  Double_t azimuthAngleLow = fAzimuthTable->At(index);
  Double_t azimuthAngleUp = fAzimuthTable->At(index+1);

  TArrayD* distanceTablePtr;
  TArrayD* ceTablePtr;

  Double_t pmtGainLow;
  distanceTablePtr = fDistanceTable;
  distanceTablePtr += index;
  ceTablePtr = fGainTable;
  ceTablePtr += index;
  Int_t index_distance = 
    Interpolation::SearchIndex(*distanceTablePtr,l,fNumberTable->At(index));

  if(TMath::Abs(distanceTablePtr->At(index_distance+1)-
		distanceTablePtr->At(index_distance))
     >1.0*kCENTI_METER){ // If Large interval
    pmtGainLow = ceTablePtr->At(index_distance)+
      (l-distanceTablePtr->At(index_distance))*
      (ceTablePtr->At(index_distance+1)-ceTablePtr->At(index_distance))/
      (distanceTablePtr->At(index_distance+1)-distanceTablePtr->At(index_distance));
  }else{
    pmtGainLow = 
      Interpolation::MThPolynominalInterpolate(*distanceTablePtr,
	    *ceTablePtr,fNumberTable->At(index),l,kDegOfPolynominal);
  }

  Double_t pmtGainUp;
  distanceTablePtr++;
  ceTablePtr++;
  index_distance = 
    Interpolation::SearchIndex(*distanceTablePtr,l,fNumberTable->At(index+1));

  if(TMath::Abs(distanceTablePtr->At(index_distance+1)-
		distanceTablePtr->At(index_distance))
     >1.0*kCENTI_METER){ // If Large interval
    pmtGainUp = ceTablePtr->At(index_distance)+
      (l-distanceTablePtr->At(index_distance))*
      (ceTablePtr->At(index_distance+1)-ceTablePtr->At(index_distance))/
      (distanceTablePtr->At(index_distance+1)-distanceTablePtr->At(index_distance));
  }else{
    pmtGainUp = 
      Interpolation::MThPolynominalInterpolate(*distanceTablePtr,
	    *ceTablePtr,fNumberTable->At(index+1),l,kDegOfPolynominal);
  }

  Double_t azimuthAngleRatio = 
    (azimuthAngle-azimuthAngleLow)/(azimuthAngleUp-azimuthAngleLow);
  Double_t pmtGain = pmtGainLow + (pmtGainUp-pmtGainLow)*azimuthAngleRatio;

  return pmtGain*fScale*fNorm;

}

