// $Id: RMOPMTChargeResponse.cxx,v 1.9 2004/10/05 19:58:29 hoshina Exp $
//*************************************************************************
//* --------------------
//* RMOPMTChargeResponse
//* --------------------
//* (Description)
//*  DOMPMTChargeResponse module is based on the charge response function.
//*  Current model of charge response function consists of gaussian component
//*  and exponential component. If you want to change the model,
//*  define your charge response function and set it to the fChargeResponse.
//*
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2003/12/22  K.Hoshina	Original version.
//*	2004/8/25   S.Yoshida	2 Dimensional Gain response included
//*************************************************************************

#include "RMOPMTChargeResponse.h"
#include "RMOPMTChargeDatum.h"
#include "RMOPMTChargeBuf.h"
#include "RMOPMTPEBuf.h"
#include "RMOUnit.h"
#include "TRandom.h"

#include <iostream>
#include <fstream>

ClassImp(RMOPMTChargeResponse)

using namespace std;

Double_t RMOPMTChargeResponse::fQt_ov_Q0  =  3.69343e-01; // qtau / q0
Double_t RMOPMTChargeResponse::fSQ0_ov_Q0 =  2.87084e-01; // sq0 / q0
Double_t RMOPMTChargeResponse::fPe        =  2.68070e-01; // pe 
Double_t RMOPMTChargeResponse::fTimeResol =  3.0 * kNANO_SECOND; // time resolution

// global function

//================================================================
// function ChargeResponse1pe_noconv
//================================================================

Double_t ChargeResponse1pe_noconv(Double_t *x, Double_t *par) 
{ 

//*-*-*-*-*-*Single p.e. Charge Response Function*-*-*-*-*-*-*-*
//           ====================================


   Double_t q0      = par[0];               // mean of gaussian term

   // scale sq0 and qtau
   Double_t sq0     = par[1] * q0; 
   Double_t qtau    = par[2] * q0; 
   Double_t pe      = par[3];
   Double_t dq0     = x[0];

   Double_t normalizelim  = - q0/sq0;
   Double_t normarize     = 0.5 * TMath::Erfc(normalizelim);

   Double_t sigexpo = pe / qtau  
                         * TMath::Exp(- dq0 / qtau);

   Double_t sum2sigma0 = sq0 * sq0;
   Double_t siggaus    = (1.- pe) * TMath::Exp(-0.5 *(dq0 - q0) * (dq0 - q0) / sum2sigma0)
                               / (TMath::Sqrt(2 * TMath::Pi() * sum2sigma0) * normarize);

   //if (x[0] > q0) sigexpo = 0.; 

   Double_t signal     = sigexpo + siggaus;

   return   signal; 

}


//=====================================================================
//* constructor -------------------------------------------------------

RMOPMTChargeResponse::RMOPMTChargeResponse()
                   : RMOVModule("RMOPMTChargeResponse","pe to charge simulator"),
                     fChargeResponse(0), fChargeHist(0), fChargeBuf(0),
                     fCanvas(0), fCanvas2(0)
{

  fGainDataName = "default";
  fGain = 5.0e7;

}
RMOPMTChargeResponse::RMOPMTChargeResponse(TString& pmtGainFileName)
                   : RMOVModule("RMOPMTChargeResponse","pe to charge simulator"),
                     fGain(0),
                     fChargeResponse(0), fChargeHist(0), fChargeBuf(0),
                     fCanvas(0), fCanvas2(0)
{

  fGainDataName = pmtGainFileName;
  fGain = 5.0e7;

}

//=====================================================================
//* destructor --------------------------------------------------------

RMOPMTChargeResponse::~RMOPMTChargeResponse()
{
//*-*-*-*-*-*-*-*-*-*-*-*Destructor*-*-*-*-*-*-*-*-*-*-*-*-*
//                       ==========
  if (f2DGain)         delete f2DGain;
  if (fChargeResponse) delete fChargeResponse;
  if (fChargeHist)     delete fChargeHist;
  if (fChargeBuf)      delete fChargeBuf;
  if (fCanvas)         delete fCanvas;
  if (fCanvas2)        delete fCanvas2;
}

//===================================================================
//* Configure ---------------------------------------------------------

void RMOPMTChargeResponse::Configure()
{
  if(fGainDataName=="default"){
    f2DGain = new RMOPMT2DGain(); // 2D Gain object
  }else{
    f2DGain = new RMOPMT2DGain(fGainDataName);
  }
   CreateSinglePEChargeResponse(); 
}

//=====================================================================
//* Process -----------------------------------------------------------

void RMOPMTChargeResponse::Process()
{
//*-*-*-*-*-*-*-*-*-*-*-*Process*-*-*-*-*-*-*-*-*-*-*-*-*
//                       =======

  /* Check out the state of task... make sure PEs exist */
  if(RMOVModule::CurrentState()!=kALIVE){
    cerr << "No PEs survive!" << endl;
    cerr << "Skip processing in the charge converting" << endl;
    ChangeState(kABSORBED);
    return;
  }

   /* Get Input Data --------------------- */

   RMOPMTPEBuf pebuf = (RMOPMTPEBuf &)GetImputBuffer();
   TIter       iter  = pebuf.GetIter();

   /* Convert pe to charge --------------- */
   if (fChargeBuf) delete fChargeBuf;

   fChargeBuf = new RMOPMTChargeBuf();
   fChargeBuf->SetOwner();

   RMOElectron *pe;
   while ((Int_t)(pe = (RMOElectron *)iter.Next())) {
      Double_t time = pe->GetTime();
      SmearTimeEdge(time);
      // the photon is alive!   
      Double_t cosTheta   = pe->GetCosTheta(); 
      Double_t alpha      = pe->GetAzimuth();
      Double_t relativeGain = f2DGain->Get2DGain(alpha, cosTheta);
      Double_t charge = fChargeResponse->GetRandom();
      RMOPMTChargeDatum *qdatum = new RMOPMTChargeDatum(charge*relativeGain, time);
      qdatum->SetMother(pe);
      fChargeBuf->Add(qdatum);

   }
   
}

//=====================================================================
//* Finish ------------------------------------------------------------

void RMOPMTChargeResponse::Finish()
{


}

//=====================================================================
//* Draw --------------------------------------------------------------

void RMOPMTChargeResponse::Draw()
{
   if (fCanvas)     delete fCanvas;
   if (fCanvas2)    delete fCanvas2;
   if (fChargeHist) delete fChargeHist;

   Int_t    nbins = 5000;
   Double_t xmax  = kCHARGE* fGain * 3.;

   fChargeHist = new TH1D("GainedCharge", "Gained charge", nbins, 0, xmax); 

   TIter iter = fChargeBuf->GetIter();
   RMOPMTChargeDatum *datum;
   while ((Int_t)(datum = (RMOPMTChargeDatum *)iter.Next())) {
      fChargeHist->Fill(datum->GetCharge());
   }
      
   fCanvas  = new TCanvas("SinglePEChargeResponse", "Charge Response of single P.E",
                          10, 10, 1200, 400);
   fCanvas2 = new TCanvas("SinglePEChargeResponse2", "Charge histogram of SPEs",
                          10, 10, 400, 400);

   fCanvas->Divide(3, 1);
   fCanvas->cd(1);
   fChargeResponse->Draw();
   fCanvas->cd(2);
   fChargeResponse->DrawIntegral();
   fCanvas->cd(3);
   f2DGain->MakeMapWithoutDraw();
   f2DGain->fGainMap->GetXaxis()->SetTitle(" X [m]");
   f2DGain->fGainMap->GetYaxis()->SetTitle(" Y [m]");
   f2DGain->fGainMap->Draw("LEGO2");
   fCanvas2->cd();
   fChargeHist->Draw();
}

//=====================================================================
//* CreateSiglePeChargeResponse -----------------------------------------

void RMOPMTChargeResponse::CreateSinglePEChargeResponse()
{
   if (GetVerboseLevel() == kFUNCTION_CALLED) {
      cerr << kFUNCTION_CALLED 
           << " RMOPMTChargeResponse::CalcSinglePEChargeResponse is called!" << endl;  
   }

   if (fChargeResponse) delete fChargeResponse;

   Double_t par[4];
   par[0] = fGain*kCHARGE;
   par[1] = fSQ0_ov_Q0; 
   par[2] = fQt_ov_Q0; 
   par[3] = fPe; 

   Int_t    nbins = 5000;
   Double_t xmax  = kCHARGE*fGain * 3.;

   fChargeResponse = new TF1("SinglePEChargeResponse", ChargeResponse1pe_noconv,
                              0, xmax, 4);
   fChargeResponse->SetLineWidth(4);
   fChargeResponse->SetLineColor(kRed);
   fChargeResponse->SetNpx(nbins);
   fChargeResponse->SetParameters(par);
   
}


//=====================================================================
//* SmearTimeEdge -----------------------------------------------------

void RMOPMTChargeResponse::SmearTimeEdge(Double_t &time)
{
   Double_t smeard = gRandom->Gaus(time, fTimeResol);    
   time = smeard;
}
