// $Id: RMOPMTPhotoCathode.cxx,v 1.8 2004/10/04 03:35:21 hoshina Exp $
//*************************************************************************
//* --------------------
//* RMOPMTPhotoCathode
//* --------------------
//* (Description)
//*	Photo Cathode module
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2003/12/22  K.Hoshina	Original version.
//*     2004/8/23   S.Yoshida   The wavelength and photon hit location 
//*                             dependences are considered.
//*************************************************************************

#include <TRandom.h>
#include <TMath.h>

#include "RMOPMTPhotoCathode.h"
#include "RMOQuickPhotonPropagator.h"
#include "RMOOpticalPhoton.h"
#include "RMOElectron.h"
#include "RMOPMTPEBuf.h"
#include "RMOPhotonBuf.h"
#include "RMOUnit.h"

//#define __RMOABSOLUTEQE__  0.3
//#define __RMOABSOLUTEQE__  1 
//#define __RMOZOFFSET__     (8.4 * kMILLI_METER)     // 8.4mm 
//#define __RMOPMTR__        (136.69 * kMILLI_METER)  // 136.69mm 

const Int_t    RMOPMTPhotoCathode::kNXbins = 30;
const Int_t    RMOPMTPhotoCathode::kNYbins = 30;
const Double_t RMOPMTPhotoCathode::kXmin   = (-15. * kCENTI_METER);
const Double_t RMOPMTPhotoCathode::kXmax   = (15. * kCENTI_METER);
const Double_t RMOPMTPhotoCathode::kYmin   = (-15. * kCENTI_METER);
const Double_t RMOPMTPhotoCathode::kYmax   = (15. * kCENTI_METER);

ClassImp(RMOPMTPhotoCathode)

using namespace std;

//=====================================================================
//* constructor -------------------------------------------------------

RMOPMTPhotoCathode::RMOPMTPhotoCathode()
                   : RMOVModule("RMOPMTPhotoCathode","Photon to pe simulator"), 
		     f2DCE(0), fQe(0), fPEMap(0), 
                     fPEBuf(0),fCanvas(0)
{

  fCeDataName = "default";
}

RMOPMTPhotoCathode::RMOPMTPhotoCathode(TString& pmtCEFileName)
                   : RMOVModule("RMOPMTPhotoCathode","Photon to pe simulator"), 
		     f2DCE(0), fQe(0), fPEMap(0), 
                     fPEBuf(0),fCanvas(0)
{
  fCeDataName = pmtCEFileName;
}

//=====================================================================
//* destructor --------------------------------------------------------

RMOPMTPhotoCathode::~RMOPMTPhotoCathode()
{
  if (f2DCE)          delete f2DCE;
  if (fQe)            delete fQe;
  if (fPEMap)         delete fPEMap;
  if (fPEBuf)         delete fPEBuf;
  if (fCanvas)        delete fCanvas;
}

//===================================================================
//* Configure ---------------------------------------------------------

void RMOPMTPhotoCathode::Configure()
{

  if(fCeDataName=="default"){
    f2DCE = new RMOPMT2DCollectEff(); // Collection Efficiency object
  }else{
    f2DCE = new RMOPMT2DCollectEff(fCeDataName);
  }

  fQe = new RMOPMTQeDatum();        // PMT Quntam Efficiency object

}

//=====================================================================
//* Process -----------------------------------------------------------

void RMOPMTPhotoCathode::Process()
{

   /* Get Input Data ------------------------ */

   RMOPhotonBuf photonbuf = (RMOPhotonBuf &)GetImputBuffer();
   TIter        iter      = photonbuf.GetIter(); 

   /* Convert photon to pe ------------------ */
  
   if (fPEBuf) delete fPEBuf;

   fPEBuf = new RMOPMTPEBuf(); 
   fPEBuf->SetOwner();

   RMOOpticalPhoton *ph; 

   TFile *calibfile = TFile::Open(RMOQuickPhotonPropagator::GetCalibRootFileName());

   while ((Int_t)(ph = (RMOOpticalPhoton *)iter.Next())) {
      if (ph->IsValid()) {
         // the photon is alive!   
         Double_t cosTheta   = ph->GetCosTheta(); 
         Double_t alpha      = ph->GetAzimuth();
	 Double_t lambda     = ph->GetWaveLength();

	 // PMT to pe conversion efficiencies
         Double_t collect_eff = f2DCE->Get2DCollectEff(alpha, cosTheta);

         while (collect_eff == 0) {
            // tmp treatment! <<<<<<<<<<<<<<<<<<<<<<<
            // current version of ROMEO cannot loss photon!
            cerr << "***********************************************" << endl;
            cerr << "RMOPMTPhotoCathode::Process: collect_eff = " << collect_eff << endl;
            RMOOpticalPhoton *origph = (RMOOpticalPhoton *)ph->GetMother();
            TString dirname(RMOQuickPhotonPropagator::GetCalibRootFileName());
            dirname += ":/";
            TVector3 localpos = RMOQuickPhotonPropagator::GetRandomPosition(
                                       origph,
                                       dirname, 
                                       kFALSE, GetVerboseLevel());
            cosTheta = localpos.CosTheta();
            alpha    = localpos.Phi();
            while (alpha < 0)              alpha += TMath::TwoPi();    // 0 < phi < 2*pi
            while (alpha > TMath::TwoPi()) alpha -= TMath::TwoPi();
            ph->SetPosition(localpos);
            ph->SetCosTheta(cosTheta);
            ph->SetAzimuth(alpha);

            cerr << "cos, alpha = " << cosTheta << " " << alpha << endl;
            collect_eff = f2DCE->Get2DCollectEff(alpha, cosTheta);
         }

	 Double_t qe = fQe->GetQe(lambda);

         Double_t prob   = qe*collect_eff/kPERCENT; // [% x %]/[%] = [%]
         Double_t rand   = gRandom->Uniform(0., 1.) * kPERCENT; // percent

         TString err("RMOPMTPhotoCathode::Process: cos, alpha, p, rand = ");
         ((((((err += cosTheta) += " ") += alpha*TMath::RadToDeg()) 
	    += " ") += prob) += " ") += rand;
         VerbosePrint(err, kDEFAULT);

         if (rand < prob) {
            VerbosePrint(TString("Congraturation!!! 1 p.e generated!")
                                 , kDEFAULT); 
            RMOElectron *pe = new RMOElectron();
	    pe->SetAzimuth(alpha);
	    pe->SetCosTheta(cosTheta);

            pe->SetMother(ph);
            fPEBuf->Add(pe); 
         }
      }      
   }
   calibfile->Close();

   /* If all photons are absorbed, change state - */
   if (fPEBuf->GetEntries() == 0) {
      cerr << "RMOPMTPhotoCathode::Process: Sorry! "
           << "All photons are absorbed. " << endl;
      ChangeState(kABSORBED);
      return;
   }
   
}

//=====================================================================
//* Finish ------------------------------------------------------------

void RMOPMTPhotoCathode::Finish()
{
}

//=====================================================================
//* Draw --------------------------------------------------------------

void RMOPMTPhotoCathode::Draw()
{
   if (fCanvas) delete fCanvas;
   if (fPEMap)  delete fPEMap;
   if (!f2DCE){
     cerr << "RMOPMT2DCoolectEff object not generated, yet" << endl;
     abort();
   }

   fCanvas = new TCanvas("PE hit location Map", "PE", 
                          10, 10, 800, 400);

   fPEMap  = new TH2D("PhotoElectronMap", "PhotoElectronMap",
                               kNXbins, kXmin, kXmax, 
                               kNYbins, kYmin, kYmax );

   TIter iter = fPEBuf->GetIter(); 
   RMOElectron *pe;
   while ((Int_t)(pe = (RMOElectron *)iter.Next())) {
     Double_t cosTheta   = pe->GetCosTheta(); 
     Double_t alpha      = pe->GetAzimuth();
     Double_t d = f2DCE->CosZenithToDistance(cosTheta);
     Double_t x = d*TMath::Cos(alpha);
     Double_t y = d*TMath::Sin(alpha);
     fPEMap->Fill(x, y);
  }

   fCanvas->cd();
   fPEMap->GetXaxis()->SetTitle("X[m]");
   fPEMap->GetYaxis()->SetTitle("Y[m]");
   fPEMap->Draw("LEGO2");

   f2DCE->Draw();

}

