//*************************************************************************
//* ---------------------
//* RMOPMTQeDatum Class
//* ---------------------
//* (Description)
//*     module class for PMT Qe
//* (Requires)
//* (Provides)
//*	RMOPMTQeDatum              : Qe modules
//* (Update Record)
//*	2004/8/14  S.Yoshida	Original version.
//*************************************************************************

#include "RMOPMTQeDatum.h"
#include "Interpolation.h"

const Int_t    RMOPMTQeDatum::kNXbins = 120;
const Double_t RMOPMTQeDatum::kXmin   = (200. * kNANO_METER);
const Double_t RMOPMTQeDatum::kXmax   = (800. * kNANO_METER);

const  Int_t RMOPMTQeDatum::kDegOfPolynominal = 6;
 

ClassImp(RMOPMTQeDatum)

using namespace std;

//=====================================================================
//* constructor -------------------------------------------------------

RMOPMTQeDatum::RMOPMTQeDatum(TString& fileName)
  : RMOVDatum(), fQeTable(0), fWaveLengthTable(0),fQeMap(0), fCanvas(0)
{
   fQeDataName = fileName;
   fScale = 1.0;
   ReadQeTable();
}

//=====================================================================
//* destructor --------------------------------------------------------

RMOPMTQeDatum::~RMOPMTQeDatum()
{
  if (fQeTable)      delete fQeTable;
  if (fWaveLengthTable)   delete fWaveLengthTable;
  if (fCanvas) delete fCanvas;
  if (fQeMap) delete fQeMap;

}

//=====================================================================
//* Draw --------------------------------------------------------------

void RMOPMTQeDatum::Draw()
{
  if (fCanvas) delete fCanvas;
  if (fQeMap)  delete fQeMap;

   fCanvas = new TCanvas("fQeTable_QeMap", "IceCube PMT Qe", 
                          10, 10, 800, 400);

   fQeMap  = new TH1D("QeMap", "QeMap",
		      kNXbins, kXmin, kXmax);

   Double_t epsilon = 1.0e-10; // Adjustment for binning

   for(Int_t iLambda = 0;iLambda<kNXbins;iLambda++){
     Double_t lambda = kXmin + 5.0*((Double_t)iLambda)*kNANO_METER + epsilon;//[nm]
     Double_t qe = GetQe(lambda);

     Int_t bin = fQeMap->FindBin(lambda);
     Double_t content = fQeMap->GetBinContent(bin);
     if(content){
       qe += content;
       qe = qe/2.0;
       fQeMap->SetBinContent(bin,qe);
     }else{
       fQeMap->Fill(lambda,qe);
     }
   }

   fQeMap->GetXaxis()->SetTitle(" Wave Length [m]");
   fQeMap->GetYaxis()->SetTitle(" Qe [%]");
   fCanvas->cd();
   fQeMap->Draw("C");

}

//=====================================================================
//* Read the Qe data table ------------------------------------------
//=====================================================================
void RMOPMTQeDatum::ReadQeTable()
{
   // uniformity data open

   ifstream in(const_cast<Char_t*>(fQeDataName.Data()));

   if (!in.good()) {
      cerr << "RMOPMTQeDatum::ReadQeTable: file " << fQeDataName
           << " open failed! abort." << endl;
      abort();
   } 

   Double_t  lambda, qe;

   fQeTable = new TArrayD(kNumber_of_data);
   fWaveLengthTable = new TArrayD(kNumber_of_data);

   Int_t index = 0;
   while (in >> lambda >> qe) {
     fWaveLengthTable->AddAt(lambda*kNANO_METER,index);
     fQeTable->AddAt(qe,index);
     cerr << "lambda(" << fWaveLengthTable->At(index) << ") Qe(" 
        	  << fQeTable->At(index) << ") " << endl;

     index++;

   }

   in.close();
   in.clear();
}

//=====================================================================
// Get Quantum Efficieny---------------------------------------------
// lambda: Wave length [m]
//=====================================================================
Double_t RMOPMTQeDatum::GetQe(Double_t lambda)
{


  Double_t qe= 0.0;

  if(fWaveLengthTable){
    Int_t m;
    if(lambda<290.0*kNANO_METER) m=2;
    else m = kDegOfPolynominal;
    qe = 
    Interpolation::MThPolynominalInterpolate(*fWaveLengthTable, *fQeTable,
					     kNumber_of_data, lambda, m);

    if(lambda<240.0*kNANO_METER || lambda> 720.0*kNANO_METER) qe=0.0;
    if(qe<0.0) qe=0.0;

  }else{
    cerr << "Qe table has not been read out, yet!" << endl;
    abort();
  }
 

   return qe*fScale;

}

