// $Id: RMOPMTWaveForm.cxx,v 1.9 2004/10/05 19:58:29 hoshina Exp $
//*************************************************************************
//* --------------------
//* RMOPMTWaveForm
//* --------------------
//* (Description)
//*     Wave form module class. Generate single p.e pulse and store them
//*     to the RMOPMTWaveFormBuf. 
//*     Note that pulse integration is performed by RMOPMTWaveFormBuf.
//*	
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2003/12/22  K.Hoshina	Original version.
//*************************************************************************

#include "RMOPMTWaveForm.h"
#include "RMOPMTWaveFormBuf.h"
#include "RMOPMTSingleWaveForm.h"
#include "RMOPMTChargeBuf.h"
#include "RMOPMTChargeDatum.h"
#include "RMOUnit.h"

#include <iostream>

ClassImp(RMOPMTWaveForm)

using namespace std;


//=====================================================================
//* constructor -------------------------------------------------------

RMOPMTWaveForm::RMOPMTWaveForm( const Char_t  *name, 
                                const Char_t  *title)
                   : RMOVModule(name,title), fNormHist(0), fWaveFormBuf(0),
                     fCanvas(0), fCanvas1(0)
{
   fSigma      = (2.18659 * kNANO_SECOND); // ns
   fTerminator = (50 * kOHM);             // 50 ohm
   fT0         = (20 * kNANO_SECOND);      // 10 ns 
   fGround     = (0 * kVOLT);             // ground level
}

//=====================================================================
//* destructor --------------------------------------------------------

RMOPMTWaveForm::~RMOPMTWaveForm()
{
   if (fNormHist)    delete fNormHist;
   if (fWaveFormBuf) delete fWaveFormBuf;
   if (fCanvas)      delete fCanvas;
   if (fCanvas1)     delete fCanvas1;
}

//=====================================================================
//* CalcNormalizationOfSingleWaveForm --------------------------------------------------

Double_t RMOPMTWaveForm::CalcNormalizationOfSingleWaveForm(Double_t charge)
{
   
   // V[V] = I[C/s] * R[Ohm];
   // integV = integ(I[C/s] * R[Ohm] ds) = Normarization;
   // K = (C * R) / (V.s) 
   // 

   //Double_t norm = (charge * fTerminator) / (kMILLI_VOLT * kNANO_SECOND) ; 
   Double_t norm = (charge * fTerminator); 
   TString err("RMOPMTWaveForm:: charge, current normalization = "); 
   err += charge;
   err += " ";
   err += norm;
   VerbosePrint(err, kPRECISE);
   return norm; 
}

//===================================================================
//* Configure ---------------------------------------------------------

void RMOPMTWaveForm::Configure()
{

}

//=====================================================================
//* Process -----------------------------------------------------------

void RMOPMTWaveForm::Process()
{

   /* Get Input Data ----------------------------- */

   RMOPMTChargeBuf qbuf = (RMOPMTChargeBuf &)GetImputBuffer();
   TIter           iter = qbuf.GetIter();

   /* Create single pulse from charge datum ------ */

   if (fWaveFormBuf) delete fWaveFormBuf;

   fWaveFormBuf = new RMOPMTWaveFormBuf();
   fWaveFormBuf->SetOwner();
   fWaveFormBuf->SetNpx(5000);

   Double_t wfpar = 1.;            // normalization of superimposed function
   fWaveFormBuf->SetParameters(&wfpar);

   RMOPMTChargeDatum *chdatum;
   RMOPMTSingleWaveForm *wfdatum;
   Int_t i = 0;

   while ((Int_t)(chdatum = (RMOPMTChargeDatum *)iter.Next())) {
            
      Double_t charge = chdatum->GetCharge();
      Double_t time   = chdatum->GetTime();
      Double_t norm   = CalcNormalizationOfSingleWaveForm(charge);
      Double_t par[4];
      par[0] = norm;
      par[1] = fSigma; 
      par[2] = fT0 + time;
      par[3] = fGround; 

      TString fn("WaveForm_pulse");
      fn += i;
      wfdatum = new RMOPMTSingleWaveForm(fn);
      wfdatum->SetLineWidth(2);
      wfdatum->SetLineColor(kBlue);
      wfdatum->SetNpx(500);
      wfdatum->SetParameters(par);
      wfdatum->SetMother(chdatum);
      fWaveFormBuf->Add(wfdatum);
      
      i++;

   } 

}

//=====================================================================
//* Finish ------------------------------------------------------------

void RMOPMTWaveForm::Finish()
{

   if (GetVerboseLevel() >= kPRECISE) {
      TIter iter = fWaveFormBuf->GetIter(); 
      RMOPMTSingleWaveForm *wf;
      while ((Int_t)(wf = (RMOPMTSingleWaveForm *)iter.Next())) {
         wf->DumpInfo();
      }
      fWaveFormBuf->DumpInfo();
   }

}

//=====================================================================
//* Draw --------------------------------------------------------------

void RMOPMTWaveForm::Draw()
{
   // temporary version!!!

   fCanvas = new TCanvas("WaveForm", "WaveForm of single pe", 10, 10, 600, 400);
   fCanvas->cd();
   fNormHist = new TH1D("Normalization" , "Normalization of simulated pulse", 
                         500, 0, 5000);

   TIter iter = fWaveFormBuf->GetIter(); 
   RMOPMTSingleWaveForm *wf;
   Bool_t isfirst = kTRUE;
   while ((Int_t)(wf = (RMOPMTSingleWaveForm *)iter.Next())) {
      if (isfirst) {
         wf->SetMinimum(-0.3);
         wf->SetMaximum(0.1);
         wf->Draw();
         isfirst = kFALSE;
      } else {
         wf->Draw("same");
      }
      Double_t norm = wf->GetParameter(1); //normarization
      fNormHist->Fill(norm);
   }

   fCanvas1 = new TCanvas("Sumup_pulse", "Sumup_pulse", 10, 10, 400, 400);
   fCanvas1->cd();
   fWaveFormBuf->Draw();

}

