// $Id: RMOPMTWaveFormBuf.cxx,v 1.5 2004/10/05 19:58:29 hoshina Exp $
//*************************************************************************
//* --------------------
//* RMOPMTWaveFormBuf
//* --------------------
//* (Description)
//*     Buffer class for single p.e. waveforms. 
//*     RMOPMTWaveFormBuf itself has TF1 methods and its charasteristic
//*     function is pulse integration(see Function(double*, double*)). 
//*     Stored single p.e. waveforms are summed up by this class.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2003/12/22  K.Hoshina	Original version.
//*************************************************************************

#include "RMOPMTWaveFormBuf.h"
#include "RMOPMTSingleWaveForm.h"

ClassImp(RMOPMTWaveFormBuf)

using namespace std;

Double_t RMOPMTWaveFormBuf::fThreshold = (-20. * kMILLI_VOLT); // 

//=====================================================================
//* constructor -------------------------------------------------------
RMOPMTWaveFormBuf::RMOPMTWaveFormBuf( const Char_t  *name,
                                      Double_t xmin, Double_t xmax,
                                      Int_t npar)
                  :RMOVFunction(name, xmin, xmax, npar), RMOVAttBuffer(),
                   fIsSaturate(kFALSE), fMainPeak(0)
{
//*-*-*-*-*-*-*-*-*-*-*-*Constructor*-*-*-*-*-*-*-*-*-*-*-*-*
//                       ===========

}

//=====================================================================
//* Function -------------------------------------------------------

Double_t RMOPMTWaveFormBuf::Function(Double_t *x, Double_t *par)
{
//*-*-*-*-*-*Function of suprerimpose single pulses*-*-*-*-*-*-*-*
//           ======================================
   
   Double_t norm = 1;
   Double_t val = 0;
   TIter iter = GetIter();
   RMOPMTSingleWaveForm *wf;

   while ((Int_t)(wf = (RMOPMTSingleWaveForm *)iter.Next())) {
      if (wf->IsValid()) {
         Double_t curval = wf->Eval(*x); 
         val += curval;
      }
   }   

   return (norm * val);

}

//=====================================================================
//* DrawCopy ----------------------------------------------------------

TF1* RMOPMTWaveFormBuf::DrawCopy(Option_t *option) const
{
//*-*-*-*Draw a copy of this function with its current attributes*-*-*
//       ========================================================
//  See TF1::DrawCopy to get more precise descriptions.
//

   RMOPMTWaveFormBuf *newf1 = new RMOPMTWaveFormBuf();
   Copy(*newf1);
   newf1->AppendPad(option);
   newf1->SetBit(kCanDelete);
   return newf1;
}

//=====================================================================
//* GetTOT ------------------------------------------------------------

void RMOPMTWaveFormBuf::GetTOTs(Double_t time[])
{
// Return array of Time Over Threshold(TOT).
// Order of the TOT is same as the order of Cluster.
// Array size of time must be bigger than GetNsubpeakss(). 

   if (!fMainPeak) DivideToSubpeaks();
   for (Int_t i=0; i<GetNsubpeaks(); i++) {
      time[i] = ((SubpeakDatum *)fSubpeaks[i])->GetTOT();
   }
}

//=====================================================================
//* DivideToSubpeaks -------------------------------------------------------

void RMOPMTWaveFormBuf::DivideToSubpeaks()
{
// Divide a waveform into several subpeaks.
// Subpeak is a part of waveform which amplitude
// is continuously bigger than the fThreshold :
// its start/end edges are intersections between
// waveform and fThreshold. 
// NOTE:
// If threshold is bigger than ground, threshold
// is shifted to fThreshold - 1.* kMICRO_VOLT.  

   Double_t x, y;
   Double_t startedge = 0;
   Double_t endedge = 0;
   Double_t xmax = GetXmax();
   Double_t xmin = GetXmin();
   Double_t dx   = (xmax - xmin) / GetNpx();

   Bool_t foundstartedge = kFALSE;

   Double_t thres = fThreshold;
   y = Eval(xmin);
   if (thres >= y) thres -= 1. * kMICRO_VOLT;

   for (Int_t i=0; i<GetNpx(); i++) {
      x = xmin + (i + 0.5) * dx;
      y = Eval(x);

      if (y < thres) {
         if (!foundstartedge) {
            foundstartedge = kTRUE;
            startedge = GetX(thres, x - dx, x + dx); 
         }

      } else {
         if (foundstartedge) {
            // one cluster found!
            endedge = GetX(thres, x - dx, x + dx); 
            SubpeakDatum *datum = new SubpeakDatum(this, startedge, endedge);
            fSubpeaks.Add(datum); 
            foundstartedge = kFALSE;
         }
      }
   }

   if (fSubpeaks.GetEntries() == 0) {
      SubpeakDatum *datum = new SubpeakDatum(this, xmin, xmax);
      fSubpeaks.Add(datum); 
   }

   Double_t maxph = 0;
   for (Int_t i=0; i<fSubpeaks.GetEntries(); i++) {
      SubpeakDatum *datum = (SubpeakDatum *)fSubpeaks[i];
      Double_t curph = datum->GetAmplitude();
      if (maxph < curph) {
         maxph = curph;
         fMainPeak = datum;
      }
   }

}

//=====================================================================
//* DumpInfo ----------------------------------------------------------

void RMOPMTWaveFormBuf::DumpInfo(Option_t *opt)
{

   cerr << "*-* RMOPMTWaveFormBuf::DumpInfo *--------------------" << endl;
   cerr << "* Npulses       :  " << GetNpulses() << endl;
   cerr << "* Nsubpeaks     :  " << GetNsubpeaks() << endl;
   cerr << "* Threshold     :  " << fThreshold << " [V]" << endl;
   cerr << "* StartTime     :  " << GetStartTime() << " [sec]" << endl;
   cerr << "* PeakTime      :  " << GetPeakTime() << " [sec]" << endl;
   cerr << "* PeakAmplitude :  " << GetPeakAmplitude() << " [V]" << endl;
   cerr << "* Saturate      :  " << IsSaturate() << "     [0,1 : No,Yes]" << endl;

   Double_t time[GetNsubpeaks()];
   GetTOTs(time);

   for (Int_t i=0; i<GetNsubpeaks(); i++) {
      cerr << "* TOT[" << i << "]        :  " << time[i] << " [sec]" << endl;
   }

}


