// $Id: RMOPMTWaveFormBuf.h,v 1.5 2004/10/05 19:58:29 hoshina Exp $
#ifndef __RMOPMTWaveFormBuf__H__
#define __RMOPMTWaveFormBuf__H__

//*************************************************************************
//* ---------------------
//* RMOPMTWaveFormBuf Class
//* ---------------------
//* (Description)
//*     Buffer class for single p.e. waveforms.
//*     RMOPMTWaveFormBuf itself has TF1 methods and its charasteristic
//*     function is pulse integration. Stored single p.e. waveforms are
//*     summed up by this class.
//* (Requires)
//*     RMOPMTSingleWaveForm
//* (Provides)
//*	RMOPMTWaveFormBuf             
//* (Update Record)
//*	2003/12/15  K.Hoshina	Original version.
//*************************************************************************


#include "RMOVFunction.h"
#include "RMOVAttBuffer.h"
#include <iostream>

class RMOPMTWaveFormBuf : public RMOVFunction, public RMOVAttBuffer {

public:

   RMOPMTWaveFormBuf(const char* name="RMOPMTWaveFormBuf", 
                     Double_t xmin = 0, 
                     Double_t xmax = 1.0e-7,  // 100nsec
                     Int_t    npar = 1);

   virtual ~RMOPMTWaveFormBuf(){}

   virtual TF1             *DrawCopy(Option_t *option) const;
   virtual void             DumpInfo(Option_t *opt="");
   virtual Double_t         Function(Double_t *x, Double_t *par);
   inline virtual Int_t     GetNpulses()     { return GetEntries(); }  
   inline virtual Int_t     GetNsubpeaks() ;
   inline virtual Double_t  GetStartTime() ;
   inline virtual Double_t  GetPeakAmplitude() ;
   inline virtual Double_t  GetPeakTime() ;
   virtual void             GetTOTs(Double_t time[]) ;
   inline virtual Bool_t    IsSaturate()     { return fIsSaturate; }
   inline virtual void      ResetSubpeaks();
   virtual void             SavePrimitive(ofstream &out, Option_t *option) {}
   static void              SetThreshold(Double_t thres) { fThreshold = thres; }
 
protected:
  
   //------------------------------
   // protected class SubpeakDatum
   // 
   // Subpeak is a part of integrated pulse.
   // If the integrated pulse of several single p.e. pulses has
   // several valleys under the fThreshold, the integrated pulse
   // is divided into Subpeaks. The start/end point of Subpeak
   // is defined when the amplitude cross the fThreshold value.

   class SubpeakDatum : public TObject {
     public:
       SubpeakDatum() : TObject(), fFunc(0), fXmin(0), fXmax(0) {}
       SubpeakDatum(TF1 *func, Double_t xmin, Double_t xmax) 
                      : TObject(), fFunc(func), fXmin(xmin), fXmax(xmax) {}
       ~SubpeakDatum(){}

       inline Double_t GetStartTime() const { return fXmin; }
       inline Double_t GetPeakTime() const  
                       { return fFunc->GetMinimumX(fXmin, fXmax); }
       inline Double_t GetTOT() const       { return fXmax - fXmin; }
       inline Double_t GetAmplitude() const 
                       { return TMath::Abs(fFunc->GetMinimum(fXmin, fXmax)); }

     private:
       TF1*     fFunc;  
       Double_t fXmin;  
       Double_t fXmax;  
   };   

protected:

   virtual void           DivideToSubpeaks();
   virtual void           Saturate(Bool_t s = kTRUE)   { fIsSaturate = s; }
   virtual void           SetMainPeak(SubpeakDatum *d) { fMainPeak = d; }
   virtual SubpeakDatum  *GetMainPeak()                { return fMainPeak; }
   virtual TObjArray     *GetSubpeaks()                { return &fSubpeaks; }

private:

   static Double_t  fThreshold;       // threshold voltage to get start time [V]
   Bool_t           fIsSaturate;      // saturation flag
   TObjArray        fSubpeaks;        // array of subpeaks
   SubpeakDatum    *fMainPeak;        // subpeak which has most large amplitude
 
   ClassDef(RMOPMTWaveFormBuf, 1)     // Buffer class for photo electrons

};

//---------------------------
// inline functions 
//---------------------------

Int_t RMOPMTWaveFormBuf::GetNsubpeaks()
{
   if (!fMainPeak) this->DivideToSubpeaks();
   return fSubpeaks.GetEntries();
}

Double_t RMOPMTWaveFormBuf::GetStartTime()
{
   if (!fMainPeak) this->DivideToSubpeaks();
   return fMainPeak->GetStartTime();
}

Double_t RMOPMTWaveFormBuf::GetPeakAmplitude()
{
   if (!fMainPeak) this->DivideToSubpeaks();
   return fMainPeak->GetAmplitude();
}

Double_t RMOPMTWaveFormBuf::GetPeakTime()
{
   if (!fMainPeak) this->DivideToSubpeaks();
   return fMainPeak->GetPeakTime();
}

void RMOPMTWaveFormBuf::ResetSubpeaks()
{
   fSubpeaks.SetOwner();
   fSubpeaks.Delete();
   fMainPeak = 0;
}

#endif
